<?php
/*
Plugin Name: Calculated Fields Form
Plugin URI: https://cff.dwbooster.com
Description: Create forms with field values calculated based in other form field values.
Version: 5.0.214
Text Domain: calculated-fields-form
Author: CodePeople
Author URI: https://cff.dwbooster.com
License: GPL
*/

if(!defined('WP_DEBUG') || true != WP_DEBUG)
{
	error_reporting(E_ERROR|E_PARSE);
}

require_once 'inc/cpcff_session.inc.php';
// Start Session
CP_SESSION::session_start();

// Defining main constants
define('CP_CALCULATEDFIELDSF_VERSION', '5.0.214' );
define('CP_CALCULATEDFIELDSF_MAIN_FILE_PATH', __FILE__ );
define('CP_CALCULATEDFIELDSF_BASE_PATH', dirname( CP_CALCULATEDFIELDSF_MAIN_FILE_PATH ) );
define('CP_CALCULATEDFIELDSF_BASE_NAME', plugin_basename( CP_CALCULATEDFIELDSF_MAIN_FILE_PATH ) );

/* initialization / install / uninstall functions */
require_once 'inc/cpcff_auxiliary.inc.php';
require_once 'config/cpcff_config.cfg.php';

require_once 'inc/cpcff_banner.inc.php';
require_once 'inc/cpcff_main.inc.php';

// Global variables
CPCFF_MAIN::instance(); // Main plugin's object

require_once 'inc/cpcff_auto_update.inc.php';
require_once 'inc/cpcff_discounts.inc.php';
require_once 'inc/cpcff_form_cache.inc.php';

add_action( 'init', 'cp_calculated_fields_form_check_posted_data', 1 );

// functions
//------------------------------------------

function cp_calculated_fields_form_check_posted_data() {

    global $wpdb;

	$cpcff_main = CPCFF_MAIN::instance();

	if(!empty($_GET['cp_calculatedfieldsf_ipncheck']))
		cp_calculatedfieldsf_check_IPN_verification($_GET['cp_calculatedfieldsf_ipncheck']);

    if (isset( $_GET['cp_calculatedfieldsf'] ) && $_GET['cp_calculatedfieldsf'] == 'captcha' )
    {
        @include_once dirname( __FILE__ ) . '/captcha/captcha.php';
        exit;
    }

    if (isset( $_GET['cp_calculatedfieldsf_csv'] ) && is_admin() )
    {
        check_admin_referer( 'session_id_'.CP_SESSION::session_id(), '_cpcff_nonce' );
        cp_calculatedfieldsf_export_csv();
        return;
    }

    if (isset( $_GET['cp_calculatedfieldsf_export'] ) && is_admin() )
    {
        check_admin_referer( 'session_id_'.CP_SESSION::session_id(), '_cpcff_nonce' );
		$cpcff_main->get_form(intval(@$_GET['name']))->export_form();
        return;
    }

    if ( 'POST' == $_SERVER['REQUEST_METHOD'] && isset( $_POST['cp_calculatedfieldsf_post_options'] ) && is_admin() )
    {
        cp_calculatedfieldsf_save_options();
        if(
			isset($_POST['preview']) &&
			isset($_POST['cp_calculatedfieldsf_id'])
		)
		{
            print($cpcff_main->public_form(array('id' => @intval($_POST['cp_calculatedfieldsf_id']))));
			wp_footer();
			wp_die();
		}
		return;
    }

	if ( 'POST' != $_SERVER['REQUEST_METHOD'] || ! isset( $_POST['cp_calculatedfieldsf_pform_process'] ) )
	    if ( 'GET' != $_SERVER['REQUEST_METHOD'] || !isset( $_GET['hdcaptcha_cp_calculated_fields_form_post'] ) )
		    return;

    define("CP_CALCULATEDFIELDSF_ID",@$_POST["cp_calculatedfieldsf_id"]);
	$form_obj = $cpcff_main->get_form(CP_CALCULATEDFIELDSF_ID);

    if (isset($_GET["ps"])) $sequence = $_GET["ps"]; else if (isset($_POST["cp_calculatedfieldsf_pform_psequence"])) $sequence = $_POST["cp_calculatedfieldsf_pform_psequence"];
    if (!isset($_GET['hdcaptcha_cp_calculated_fields_form_post']) || $_GET['hdcaptcha_cp_calculated_fields_form_post'] == '') $_GET['hdcaptcha_cp_calculated_fields_form_post'] = @$_POST['hdcaptcha_cp_calculated_fields_form_post'];
    if (
			/**
			 * Filters applied for checking if the form's submission is valid or not
			 * returns a boolean
			 */
			!apply_filters( 'cpcff_valid_submission', true) ||
			(
				($form_obj->get_option('cv_enable_captcha', CP_CALCULATEDFIELDSF_DEFAULT_cv_enable_captcha) != 'false') &&
				( (strtolower($_GET['hdcaptcha_cp_calculated_fields_form_post']) != strtolower(CP_SESSION::get_var('rand_code'.$sequence))) ||
				  (CP_SESSION::get_var('rand_code'.$sequence) == '') )
			)
       )
    {
        echo 'captchafailed';
        exit;
    }

	// if this isn't the real post (it was the captcha verification) then echo ok and exit
    if ( 'POST' != $_SERVER['REQUEST_METHOD'] || ! isset( $_POST['cp_calculatedfieldsf_pform_process'] ) )
	{
	    echo 'ok';
        exit;
	}

	// Defines the $params array
	$params = array(
		'formid'   => CP_CALCULATEDFIELDSF_ID
	);

	// Check the honeypot
	if( ( $honeypot = get_option( 'CP_CALCULATEDFIELDSF_HONEY_POT', '' ) ) !== '' && !empty( $_REQUEST[ $honeypot ] ) )
	{
		exit;
	}

	// Check the nonce
	// Filters applied to decide if the nonce should be checked or not
	if (
		apply_filters('cpcff_check_nonce', true) &&
		(
			empty($_REQUEST['_cpcff_public_nonce']) ||
			!wp_verify_nonce($_REQUEST['_cpcff_public_nonce'], 'cpcff_form_'.CP_CALCULATEDFIELDSF_ID.$_POST["cp_calculatedfieldsf_pform_psequence"])
		)
	)
	{
		_e( 'Failed security check', 'calculated-fields-form' );
		exit;
	}

	// get form info
    //---------------------------
    $paypal_zero_payment = $form_obj->get_option('paypal_zero_payment',CP_CALCULATEDFIELDSF_DEFAULT_PAYPAL_ZERO_PAYMENT);
    require_once(ABSPATH . "wp-admin" . '/includes/file.php');

	$form_data = $form_obj->get_option( 'form_structure', CP_CALCULATEDFIELDSF_DEFAULT_form_structure );
    $fields = array();
	$choicesTxt = array();	   // List of choices texts in fields where exits
    $choicesVal = array(); // List of choices vals  in fields where exits

    foreach ($form_data[0] as $item)
        //if (!isset($item->hidefield) ||$item->hidefield != '1')
        {
            $fields[$item->name] = $item;
            if( property_exists( $item, 'choicesVal' ) && property_exists( $item, 'choices' ) )
			{
				$choicesTxt[$item->name] = $item->choices;
				$choicesVal[$item->name] = $item->choicesVal;
			}

            if ($item->ftype == 'fPhone' && isset($_POST[$item->name.$sequence])) // join fields for phone fields
            {
				$_POST[$item->name.$sequence] = '';
                for($i=0; $i<=substr_count($item->dformat," "); $i++)
                {
                    $_POST[$item->name.$sequence] .= ($_POST[$item->name.$sequence."_".$i]!=''?($i==0?'':'-').$_POST[$item->name.$sequence."_".$i]:'');
                    unset($_POST[$item->name.$sequence."_".$i]);
                }
            }
        }

	// get base price
	$request_cost = $form_obj->get_option('request_cost', CP_CALCULATEDFIELDSF_DEFAULT_COST);
	$price_item = $fields[ $request_cost ];

	$find_arr = array( ',', '.');
	$replace_arr = array( '', '.');

	if( $price_item->ftype == 'fCalculated' )
	{
		$find_arr[ 0 ] = $price_item->groupingsymbol;
		$find_arr[ 1 ] = $price_item->decimalsymbol;
	}
    elseif( $price_item->ftype == 'fcurrency' )
	{
		$find_arr[ 0 ] = $price_item->thousandSeparator;
		$find_arr[ 1 ] = $price_item->centSeparator;
	}
	elseif( $price_item->ftype == 'fnumber' || $price_item->ftype == 'fnumberds' )
	{
		$find_arr[ 0 ] = $price_item->thousandSeparator;
		$find_arr[ 1 ] = $price_item->decimalSymbol;
	}

    $price = @$_POST[ $request_cost.$_POST["cp_calculatedfieldsf_pform_psequence"] ];
	$price = preg_replace( '/[^\d\.\,]/', '', $price );
	$price = str_replace( $find_arr, $replace_arr, $price );
	$paypal_base_amount = preg_replace( '/[^\d\.\,]/', '', $form_obj->get_option( 'paypal_base_amount', 0 ) );
	$paypal_base_amount = str_replace( $find_arr, $replace_arr, $paypal_base_amount );
	$price = max( $price, $paypal_base_amount );

    // calculate discounts if any
    //---------------------------
	$price 			= CPCFF_COUPON::apply_discount(CP_CALCULATEDFIELDSF_ID, @$_POST["couponcode"], $price);
    $discount_note 	= CPCFF_COUPON::$discount_note;
    $coupon 		= CPCFF_COUPON::$coupon_applied;

    // grab posted data
    //---------------------------
    $buffer = "";

    foreach ($_POST as $item => $value)
	{
		$fieldname = str_replace($_POST["cp_calculatedfieldsf_pform_psequence"],'',$item);
        if ( array_key_exists( $fieldname, $fields ) )
        {
			// Check if the field is required and it is empty
			if(
				property_exists($fields[$fieldname],'required') &&
				!empty($fields[$fieldname]->required) &&
				$value === ''
			)
			{
				_e('At least a required field is empty', 'calculated-fields-form');
				exit;
			}

            $buffer .= stripcslashes($fields[$fieldname]->title . ": ". (is_array($value)?(implode(", ",$value)):($value))) . "\n\n";
            $params[$fieldname] = (is_array($value)) ? array_map('stripcslashes', $value) : stripcslashes($value);
        }
	}
    foreach ($_FILES as $item => $value)
	{
		$item = str_replace( $_POST["cp_calculatedfieldsf_pform_psequence"],'',$item );
		if ( isset( $fields[ $item ] ) )
        {
			$files_names_arr = array();
			$files_links_arr = array();
			$files_urls_arr  = array();
			for( $f = 0; $f < count( $value[ 'name' ] ); $f++ )
			{
				if( !empty( $value[ 'name' ][ $f ] ) )
				{
					$uploaded_file = array(
						'name' 		=> $value[ 'name' ][ $f ],
						'type' 		=> $value[ 'type' ][ $f ],
						'tmp_name' 	=> $value[ 'tmp_name' ][ $f ],
						'error' 	=> $value[ 'error' ][ $f ],
						'size' 		=> $value[ 'size' ][ $f ],
					);
					if( CPCFF_AUXILIARY::check_uploaded_file( $uploaded_file ) )
					{
						$movefile = wp_handle_upload( $uploaded_file, array( 'test_form' => false ) );
						if ( empty( $movefile[ 'error' ] ) )
						{
							$files_links_arr[] = $params[ $item."_link" ][ $f ] = $movefile["file"];
							$files_urls_arr[]  = $params[ $item."_url" ][ $f ] = $movefile["url"];
							$files_names_arr[] = $uploaded_file[ 'name' ];

							/**
							 * Action called when the file is uploaded, the file's data is passed as parameter
							 */
							do_action(
								'cpcff_file_uploaded',
								$movefile,
								array(
									'names' => &$files_names_arr,
									'links' => &$files_links_arr,
									'urls'  => &$files_urls_arr
								)
							);
						}
					}
				}
			}

			$joinned_files_names = implode( ", ", $files_names_arr );
			$buffer .= $fields[ $item ]->title . ": ". $joinned_files_names . "\n\n";
			$params[ $item ] = $joinned_files_names;
			$params[ $item."_links"] = implode( ",",  $files_links_arr );
			$params[ $item."_urls"] = implode( ",",  $files_urls_arr );
		}
	}

	if(count($params) < 2)
	{
		_e( 'The form is empty', 'calculated-fields-form' );
		exit;
	}

    $buffer_A = $buffer;
    $params["final_price"] = $price;
    $params["couponcode"] = ($coupon?$coupon->code:"");
    $params["coupon"] = ($coupon?$coupon->code.$discount_note:"");
    if (@$_POST["bccf_payment_option_paypal"] == '1')
        $params["payment_option"] = $form_obj->get_option('enable_paypal_option_yes',CP_CALCULATEDFIELDSF_PAYPAL_OPTION_YES);
    else if (@$_POST["bccf_payment_option_paypal"] == '0')
        $params["payment_option"] = $form_obj->get_option('enable_paypal_option_no',CP_CALCULATEDFIELDSF_PAYPAL_OPTION_NO);

    $to = $form_obj->get_option('cu_user_email_field', CP_CALCULATEDFIELDSF_DEFAULT_cu_user_email_field);
	$to = explode( ',', $to );
	$to_arr = array();
	foreach( $to as $index => $value )
	{
		$value .= $_POST["cp_calculatedfieldsf_pform_psequence"];
		$_POST[ $value ] = trim( @$_POST[ $value ] );
		if( !empty( $_POST[ $value ] ) ) $to_arr[] = $_POST[ $value ];
	};

    $rows_affected = $wpdb->insert( CP_CALCULATEDFIELDSF_POSTS_TABLE_NAME, array( 'formid' => CP_CALCULATEDFIELDSF_ID,
                                                                        'time' => current_time('mysql'),
                                                                        'ipaddr' => $_SERVER['REMOTE_ADDR'],
                                                                        'notifyto' => implode( ',', $to_arr ),
                                                                        'paypal_post' => @serialize($params),
                                                                        'data' =>$buffer_A .($coupon?"\n\nCoupon code:".$coupon->code.$discount_note:"") ),
																		array( '%d', '%s', '%s', '%s', '%s', '%s' )
																		);
    if (!$rows_affected)
    {
        _e( 'Error saving data! Please try again.', 'calculated-fields-form' );
        _e( '<br /><br />Error debug information: ', 'calculated-fields-form' );
		$wpdb->print_error();
        exit;
    }

    $paypal_optional = ($form_obj->get_option('enable_paypal',CP_CALCULATEDFIELDSF_DEFAULT_ENABLE_PAYPAL) == '2');

 	// save data here
    $item_number = $wpdb->insert_id;
	CP_SESSION::set_var('cp_cff_form_data', $item_number);

    if ( ( (floatval($params["final_price"]) >= 0 && !$paypal_zero_payment) || (floatval($params["final_price"]) > 0 && $paypal_zero_payment) )
          &&
          $form_obj->get_option('enable_paypal',CP_CALCULATEDFIELDSF_DEFAULT_ENABLE_PAYPAL)
          &&
          ( !$paypal_optional || (@$_POST["bccf_payment_option_paypal"] == '1') )
          && (@$_POST["bccf_payment_option_paypal"] != '0')
        )
    {
        if ($form_obj->get_option('paypal_mode',CP_CALCULATEDFIELDSF_DEFAULT_PAYPAL_MODE) == "sandbox")
            $ppurl = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
        else
            $ppurl = 'https://www.paypal.com/cgi-bin/webscr';
        if ($form_obj->get_option('paypal_notiemails', '0') == '1')
            cp_calculatedfieldsf_process_ready_to_go_reservation( $item_number, "", $params, $fields );
?>
<html>
<head><title>Redirecting to Paypal...</title></head>
<body>
<form action="<?php echo $ppurl; ?>" name="ppform3" method="post">
<input type="hidden" name="business" value="<?php echo esc_attr($form_obj->get_option('paypal_email', CP_CALCULATEDFIELDSF_DEFAULT_PAYPAL_EMAIL)); ?>" />
<?php
$paypal_item_name = $form_obj->get_option('paypal_product_name', CP_CALCULATEDFIELDSF_DEFAULT_PRODUCT_NAME).(@$_POST["services"]?": ".trim($services_formatted[1]):"").$discount_note;
foreach ($params as $item => $value)
    $paypal_item_name = str_replace('<%'.$item.'%>',(is_array($value)?(implode(", ",$value)):($value)),$paypal_item_name);
?>
<input type="hidden" name="item_name" value="<?php echo esc_attr($paypal_item_name); ?>" />
<input type="hidden" name="item_number" value="<?php echo esc_attr($item_number); ?>" />
<input type="hidden" name="email" value="<?php echo esc_attr(@$_POST[$to]); ?>" />

<?php
$paypal_recurrent = $form_obj->get_option('paypal_recurrent',CP_CALCULATEDFIELDSF_DEFAULT_PAYPAL_RECURRENT);
$paypal_recurrent_setup = $form_obj->get_option('paypal_recurrent_setup','');
$paypal_recurrent_setup_days = $form_obj->get_option('paypal_recurrent_setup_days','15');

if( strpos( $paypal_recurrent, 'field' ) !== false )
{
	if(
		!empty( $params[ $paypal_recurrent ] ) &&
		!empty( $choicesTxt[ $paypal_recurrent ] ) &&
		!empty( $choicesVal[ $paypal_recurrent ] ) &&
		( $index = array_search( $params[ $paypal_recurrent ], $choicesTxt[ $paypal_recurrent ] ) ) !== false
	) $paypal_recurrent = $choicesVal[ $paypal_recurrent ][ $index ];
}

$paypal_recurrent = intval( $paypal_recurrent );

if ( $paypal_recurrent == 0 ) { ?>
<input type="hidden" name="cmd" value="_xclick" />
<input type="hidden" name="bn" value="NetFactorSL_SI_Custom" />
<input type="hidden" name="amount" value="<?php echo esc_attr($params["final_price"]); ?>" />
<?php } else { ?>
<?php if ($paypal_recurrent_setup != '') { ?>
<input type="hidden" name="a1" value="<?php echo esc_attr($paypal_recurrent_setup); ?>">
<input type="hidden" name="p1" value="<?php echo esc_attr($paypal_recurrent_setup_days); ?>">
<input type="hidden" name="t1" value="D">
<?php } ?>
<input type="hidden" name="cmd" value="_xclick-subscriptions">
<input type="hidden" name="bn" value="NetFactorSL_SI_Custom">
<input type="hidden" name="a3" value="<?php echo esc_attr($params["final_price"]); ?>">
<input type="hidden" name="p3" value="<?php echo esc_attr($paypal_recurrent); ?>">
<input type="hidden" name="t3" value="M">
<input type="hidden" name="src" value="1">
<input type="hidden" name="sra" value="1">
<?php } ?>

<input type="hidden" name="page_style" value="Primary" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="return" value="<?php echo esc_url($form_obj->get_option('fp_return_page', CP_CALCULATEDFIELDSF_DEFAULT_fp_return_page)); ?>">
<input type="hidden" name="cancel_return" value="<?php echo esc_url($_POST["cp_ref_page"]); ?>" />
<input type="hidden" name="no_note" value="1" />
<input type="hidden" name="currency_code" value="<?php echo esc_attr(strtoupper($form_obj->get_option('currency', CP_CALCULATEDFIELDSF_DEFAULT_CURRENCY))); ?>" />
<input type="hidden" name="lc" value="<?php echo esc_attr($form_obj->get_option('paypal_language', CP_CALCULATEDFIELDSF_DEFAULT_PAYPAL_LANGUAGE)); ?>" />
<input type="hidden" name="notify_url" value="<?php echo esc_url(CPCFF_AUXILIARY::site_url().'/?cp_calculatedfieldsf_ipncheck='.$item_number); ?>" />
<input type="hidden" name="ipn_test" value="1" />
<input class="pbutton" type="hidden" value="Buy Now" /></div>
</form>
<script type="text/javascript">
document.ppform3.submit();
</script>
</body>
</html>
<?php
        exit();
    }
    else
    {
        cp_calculatedfieldsf_process_ready_to_go_reservation( $item_number, "", $params, $fields );
		$location = $form_obj->get_option('fp_return_page', CP_CALCULATEDFIELDSF_DEFAULT_fp_return_page);
        header("Location: ".$location);
        exit;
    }
}

function cp_calculatedfieldsf_check_IPN_verification( $itemnumber ) {
    global $wpdb;

	$cpcff_main = CPCFF_MAIN::instance();
	$itemnumber = intval(@$itemnumber);

    $item_name = $_POST['item_name'];
    $item_number = $_POST['item_number'];
    $payment_status = $_POST['payment_status'];
    $payment_amount = $_POST['mc_gross'];
    $payment_currency = $_POST['mc_currency'];
    $txn_id = $_POST['txn_id'];
    $receiver_email = $_POST['receiver_email'];
    $payer_email = $_POST['payer_email'];
    $payment_type = $_POST['payment_type'];
/**
	if ($payment_status != 'Completed' && $payment_type != 'echeck')
	    return;

	if ($payment_type == 'echeck' && $payment_status != 'Pending')
	    return;
*/
	$str = '';
    if (isset($_POST["first_name"])) $str .= 'Buyer: '.$_POST["first_name"]." ".$_POST["last_name"]."\n";
    if (isset($_POST["subscr_id"])) $str .= "Subscription id: ".$_POST["subscr_id"]."\n";
    if (isset($_POST["payer_email"])) $str .= 'Payer email: '.$_POST["payer_email"]."\n";
	if (isset($_POST["residence_country"])) $str .= 'Country code: '.$_POST["residence_country"]."\n";
	if (isset($_POST["payer_status"])) $str .= 'Payer status: '.$_POST["payer_status"]."\n";
	if (isset($_POST["protection_eligibility"])) $str .= 'Protection eligibility: '.$_POST["protection_eligibility"]."\n";

	if (isset($_POST["item_name"])) $str .= 'Item: '.$_POST["item_name"]."\n";
	if (isset($_POST["payment_gross"]))
	     $str .= 'Payment: '.$_POST["payment_gross"]." ".$_POST["mc_currency"]." (Fee: ".$_POST["payment_fee"].")"."\n";
	else if (isset($_POST["mc_gross"]) && isset($_POST["mc_currency"]) && isset($_POST["mc_fee"]))
	     $str .= 'Payment: '.$_POST["mc_gross"]." ".$_POST["mc_currency"]." (Fee: ".$_POST["mc_fee"].")"."\n";
	if (isset($_POST["payment_date"])) $str .= 'Payment date: '.$_POST["payment_date"];
	if (isset($_POST["payment_type"]) && isset($_POST["payment_status"])) $str .= 'Payment type/status: '.$_POST["payment_type"]."/".$_POST["payment_status"]."\n";
	if (isset($_POST["business"])) $str .= 'Business: '.$_POST["business"]."\n";
	if (isset($_POST["receiver_email"])) $str .= 'Receiver email: '.$_POST["receiver_email"]."\n";

    $myrows = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM ".CP_CALCULATEDFIELDSF_POSTS_TABLE_NAME." WHERE id=%d", $itemnumber ) );
    $params = unserialize($myrows[0]->paypal_post);
	$form_obj = $cpcff_main->get_form($myrows[0]->formid);
    if ($myrows[0]->paid == 0)
    {
		$params[ 'paypal_data' ] = $str;
		if (isset($_POST["subscr_id"])) $params[ 'subscr_id' ] = $_POST["subscr_id"];

        $wpdb->query( $wpdb->prepare( "UPDATE ".CP_CALCULATEDFIELDSF_POSTS_TABLE_NAME." SET paid=1, paypal_post=%s WHERE id=%d", serialize( $params ), $itemnumber ) );
		if (!defined('CP_CALCULATEDFIELDSF_ID'))
            define ('CP_CALCULATEDFIELDSF_ID',$myrows[0]->formid);
        if ($form_obj->get_option('paypal_notiemails', '0') != '1')
		{
			$form_data = $form_obj->get_option( 'form_structure', CP_CALCULATEDFIELDSF_DEFAULT_form_structure );
			$fields = array();
			foreach ($form_data[0] as $item) $fields[$item->name] = $item;
			cp_calculatedfieldsf_process_ready_to_go_reservation( $itemnumber, $payer_email, $params, $fields );
		}
        echo 'OK - processed';
    }
    else
        echo 'OK - already processed';

    exit();
}

function cp_calculatedfieldsf_process_ready_to_go_reservation( $itemnumber, $payer_email = "", $params = array(), $fields = array() )
{

   global $wpdb;
   $cpcff_main = CPCFF_MAIN::instance();
   $itemnumber = intval( $itemnumber );

   $myrows = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM ".CP_CALCULATEDFIELDSF_POSTS_TABLE_NAME." WHERE id=%d", $itemnumber ) );

   if (!defined('CP_CALCULATEDFIELDSF_ID'))
        define ('CP_CALCULATEDFIELDSF_ID',$myrows[0]->formid);
	$form_obj = $cpcff_main->get_form($myrows[0]->formid);
    $buffer_A = $myrows[0]->data;
    $buffer = $buffer_A;

	$fields[ 'ipaddr' ] = $myrows[0]->ipaddr;

    global $cp_calculatedfieldsf_envelope_from;
    add_filter('phpmailer_init','cp_calculatedfieldsf_envelope_from');

    // 1- Send email
    //---------------------------
    $email_data = CPCFF_AUXILIARY::parsing_fields_on_text(
        $fields,
        $params,
        $form_obj->get_option('fp_message', CP_CALCULATEDFIELDSF_DEFAULT_fp_message),
        $buffer,
        $form_obj->get_option('fp_emailformat', CP_CALCULATEDFIELDSF_DEFAULT_email_format),
        $itemnumber
    );

    if ('true' == $form_obj->get_option('fp_inc_additional_info', CP_CALCULATEDFIELDSF_DEFAULT_fp_inc_additional_info))
    {
        $email_data[ 'text' ] .="ADDITIONAL INFORMATION\n"
								 ."*********************************\n";

		$basic_data = "IP: ".$myrows[0]->ipaddr."\n"
              ."Server Time:  ".date("Y-m-d H:i:s")."\n";

		/**
		 *	Includes additional information to the email's message,
		 *  are passed two parameters: the basic information, and the IP address
		 */
		$basic_data = apply_filters( 'cpcff_additional_information',  $basic_data, $myrows[0]->ipaddr );
		$email_data[ 'text' ] .= $basic_data;
	}

    $subject = $form_obj->get_option('fp_subject', CP_CALCULATEDFIELDSF_DEFAULT_fp_subject);
    $subject = CPCFF_AUXILIARY::parsing_fields_on_text(
		$fields,
		$params,
		$subject,
		'',
		'plain text',
		$itemnumber
	);

	$from = CPCFF_AUXILIARY::parsing_fields_on_text(
				$fields,
				$params,
				preg_replace("/(fieldname\d+)\s*%>/i", "$1_value%>", $form_obj->get_option('fp_from_email', CP_CALCULATEDFIELDSF_DEFAULT_fp_from_email)),
				'',
				'plain text',
				$itemnumber
			);
	$from = $from['text'];

	$to = CPCFF_AUXILIARY::parsing_fields_on_text(
					$fields,
					$params,
					preg_replace("/(fieldname\d+)\s*%>/i", "$1_value%>", $form_obj->get_option('fp_destination_emails', CP_CALCULATEDFIELDSF_DEFAULT_fp_destination_emails)),
					'',
					'plain text',
					$itemnumber
				);
    $to = explode(
				",",
				$to['text']
			);

    if ('html' == $form_obj->get_option('fp_emailformat', CP_CALCULATEDFIELDSF_DEFAULT_email_format))
	{
		$content_type = "Content-Type: text/html; charset=utf-8\n";
	}
	else $content_type = "Content-Type: text/plain; charset=utf-8\n";

    $replyto = explode( ',', $myrows[0]->notifyto );
    if ($form_obj->get_option('fp_emailfrommethod', "fixed") == "customer" && !empty( $replyto ) )
	    $from_1 = $replyto[ 0 ];
	else
        $from_1 = $from;

    $cp_calculatedfieldsf_envelope_from = $from_1;
    if ( strpos($from_1,">") === false ) $from_1 = '"'.$from_1.'" <'.$from_1.'>';

    foreach ($to as $item)
        if (trim($item) != '')
        {
			try
			{
				wp_mail(trim($item), $subject[ 'text' ], $email_data[ 'text' ],
					"From: ".$from_1."\r\n".
					( ( !empty( $replyto ) ) ? "Reply-To: ".str_replace(' ', '', implode( ',', $replyto ))."\r\n" : "" ).
					$content_type.
					"X-Mailer: PHP/" . phpversion(), $email_data[ 'files' ] );
			}
			catch( Exception $mail_err ){}
        }

    // 2- Send copy to user
    //---------------------------
    $notifyto = explode( ',', $myrows[0]->notifyto ); // Allows send multiple notification emails.

    if ( ( !empty( $notifyto ) || $payer_email != '') && 'true' == $form_obj->get_option('cu_enable_copy_to_user', CP_CALCULATEDFIELDSF_DEFAULT_cu_enable_copy_to_user))
    {
        $email_data = CPCFF_AUXILIARY::parsing_fields_on_text(
                        $fields,
                        $params,
                        $form_obj->get_option('cu_message', CP_CALCULATEDFIELDSF_DEFAULT_cu_message),
                        $buffer_A,
                        $form_obj->get_option('cu_emailformat', CP_CALCULATEDFIELDSF_DEFAULT_email_format),
                        $itemnumber
                    );

        $subject = $form_obj->get_option('cu_subject', CP_CALCULATEDFIELDSF_DEFAULT_cu_subject);
        $subject = CPCFF_AUXILIARY::parsing_fields_on_text(
			$fields,
			$params,
			$subject,
			'',
			'plain text',
			$itemnumber
		);

		if ('html' == $form_obj->get_option('cu_emailformat', CP_CALCULATEDFIELDSF_DEFAULT_email_format))
		{
			$content_type = "Content-Type: text/html; charset=utf-8\n";
		}
		else $content_type = "Content-Type: text/plain; charset=utf-8\n";

		$cp_calculatedfieldsf_envelope_from = $from;
		if ( strpos($from,">") === false ) $from = '"'.$from.'" <'.$from.'>';

		if ( !empty( $notifyto ) )
		{
			foreach( $notifyto as $email_address )
			{
				try
				{
					wp_mail( $email_address, $subject[ 'text' ], $email_data[ 'text' ],
							"From: ".$from."\r\n".
							$content_type.
							"X-Mailer: PHP/" . phpversion());
				}
				catch( Exception $mail_err ){}
			}
		}

        if ( !in_array( $payer_email, $notifyto ) && $payer_email != '')
		{
			try
			{
				wp_mail(trim($payer_email), $subject[ 'text' ], $email_data[ 'text' ],
						"From: ".$from."\r\n".
						$content_type.
						"X-Mailer: PHP/" . phpversion());
			}
			catch( Exception $mail_err ){}
		}
    }
    remove_filter('phpmailer_init','cp_calculatedfieldsf_envelope_from');
}

function cp_calculatedfieldsf_envelope_from( $phpmailer )
{
	// Checks if the email's headers should be corrected or not
	if( !get_option( 'CP_CALCULATEDFIELDSF_EMAIL_HEADERS', false ) ) return $phpmailer;

	global $cp_calculatedfieldsf_envelope_from;

	$cp_calculatedfieldsf_envelope_from = strtolower( $cp_calculatedfieldsf_envelope_from );
	$parts = explode( '@',  $cp_calculatedfieldsf_envelope_from );
	$home_url = CPCFF_AUXILIARY::site_url();

	if(
		strtolower( $phpmailer->Mailer ) == 'smtp' ||
		count( $parts ) != 2 ||
		strpos( $home_url, $parts[ 1 ] ) === false
	) return $phpmailer;

    $phpmailer->Sender = $cp_calculatedfieldsf_envelope_from;
    $phpmailer->From = $cp_calculatedfieldsf_envelope_from;
	return $phpmailer;
}

function cp_calculatedfieldsf_sorting_fields_in_containers( $fields_list )
{
	$new_fields_list = array();
	while( count( $fields_list ) )
	{
		$field = array_shift( $fields_list );
		$fieldType = strtolower( $field->ftype );

		if( $fieldType == 'ffieldset' || $fieldType == 'fdiv' )
		{
			$fields = $field->fields;
			if( !empty( $fields ) )
			{
				$tmp_list = array();
				$tmp_counter = 0;
				foreach( $fields as $index => $fieldName )
				{
					for( $i = 0; $i < count( $fields_list ); $i++ )
					{
						if( $fieldName == $fields_list[ $i ]->name )
						{
							$tmp_list[ $tmp_counter ] = array_splice( $fields_list, $i, 1 );
							$tmp_list[ $tmp_counter ] = $tmp_list[ $tmp_counter ][ 0 ];
							$tmp_counter++;
							break;
						}
					}
				}
				$fields_list = array_merge( $tmp_list, $fields_list );
			}
		}
		else
		{
			$new_fields_list[] = $field;
		}
	}
	return $new_fields_list;
}

function cp_calculatedfieldsf_export_csv ()
{
	$toExclude = array( 'fcommentarea', 'fsectionbreak', 'fpagebreak', 'fsummary', 'fmedia', 'ffieldset', 'fdiv', 'fbutton' );

    if (!is_admin())
        return;
    global $wpdb;
	$cpcff_main = CPCFF_MAIN::instance();
    if (!defined('CP_CALCULATEDFIELDSF_ID'))
        define ('CP_CALCULATEDFIELDSF_ID',intval($_GET["cal"]));
	$form_obj = $cpcff_main->get_form(intval($_GET['cal']));
    $headers = array( "Form ID",  "Submission ID",  "Time",  "IP Address",  "email",  "Paid",  "Final Price",  "Coupon" );
	$fields = array( 0, 1, 2, 3, 4, 5, 6, 7 );
    $values = array();
	$form_data = $form_obj->get_option( 'form_structure', CP_CALCULATEDFIELDSF_DEFAULT_form_structure );
	$fields_list = cp_calculatedfieldsf_sorting_fields_in_containers( $form_data[ 0 ] );

	// Get headers and fields
	for( $i = 0; $i < count( $fields_list ); $i++ )
	{
		$field = $fields_list[ $i ];
		$fieldType = strtolower( $field->ftype );
		if( !in_array( $fieldType, $toExclude ) )
		{
			$fields[] 	 = $field->name;
			$headers[] = ( !empty( $field->shortlabel ) ) ? $field->shortlabel : ( ( !empty( $field->title ) ) ? $field->title : $field->name );
		}
	}

	// Get rows
    $cond = '';
    if ($_GET["search"] != '') $cond .= " AND (data like '%".esc_sql($_GET["search"])."%' OR paypal_post LIKE '%".esc_sql($_GET["search"])."%')";
    if ($_GET["dfrom"] != '') $cond .= " AND (`time` >= '".esc_sql($_GET["dfrom"])."')";
    if ($_GET["dto"] != '') $cond .= " AND (`time` <= '".esc_sql($_GET["dto"])." 23:59:59')";
    if (CP_CALCULATEDFIELDSF_ID != 0) $cond .= " AND formid=".CP_CALCULATEDFIELDSF_ID;

	$events_query = "SELECT * FROM ".CP_CALCULATEDFIELDSF_POSTS_TABLE_NAME." WHERE 1=1 ".$cond." ORDER BY `time` DESC";
	/**
	 * Allows modify the query of messages, passing the query as parameter
	 * returns the new query
	 */
	$events_query = apply_filters( 'cpcff_csv_query', $events_query );
	$events = $wpdb->get_results( $events_query );

    foreach ($events as $item)
    {

        $data = array();
        $data = @unserialize( $item->paypal_post );
		if( $data === false ) continue;

        $value = array( $item->formid, $item->id, $item->time, $item->ipaddr, $item->notifyto, ( $item->paid ? "Yes" : "No" ), @$data[ "final_price" ], @$data[ "coupon" ] );

        unset($data["final_price"]);
        unset($data["coupon"]);

		$value = array_merge( $value, $data );
		$values[] = $value;
    }

    header("Content-type: application/octet-stream");
    header("Content-Disposition: attachment; filename=export.csv");

	// Print headers
    foreach ( $headers as $header )
        echo '"'.str_replace( '"', '""', $header ).'",';
    echo "\n";

	// Print rows
    foreach ( $values as $item )
    {
        foreach( $fields as $field )
		{
			if ( !isset( $item[ $field ] ) )
                $item[ $field ] = '';

            if( isset( $item[ $field.'_url' ] ) )
				$field.='_url';

            if ( is_array( $item[ $field ] ) )
                $item[ $field ] = implode( $item[ $field ], ',' );

            echo '"' . str_replace( '"', '""', $item[ $field ] ) . '",';
		}
		echo "\n";
    }

    exit;
}

function cp_calculatedfieldsf_save_options()
{
    check_admin_referer( 'session_id_'.CP_SESSION::session_id(), '_cpcff_nonce' );
    global $wpdb;
    if (!defined('CP_CALCULATEDFIELDSF_ID'))
        define ('CP_CALCULATEDFIELDSF_ID',$_POST["cp_calculatedfieldsf_id"]);

	$error_occur = false;
    if( isset( $_POST[ 'form_structure' ] ) )
    {
		// Remove bom characters
		$_POST[ 'form_structure' ] = CPCFF_AUXILIARY::clean_bom($_POST[ 'form_structure' ]);
		$form_structure_obj = CPCFF_AUXILIARY::json_decode( $_POST[ 'form_structure' ] );
		if( !empty( $form_structure_obj ) )
		{
			global $cpcff_default_texts_array;
			$cpcff_text_array = '';

			$_POST = CPCFF_AUXILIARY::stripcslashes_recursive($_POST);
			if( isset( $_POST[ 'cpcff_text_array' ] ) ) $_POST['vs_all_texts'] = $_POST[ 'cpcff_text_array' ];

			$cpcff_main = CPCFF_MAIN::instance();
			if( $cpcff_main->get_form($_POST["cp_calculatedfieldsf_id"])->save_settings($_POST) === false )
			{
				global $cff_structure_error;
				$cff_structure_error = __('<div class="error-text">The data cannot be stored in database because has occurred an error with the database structure. Please, go to the plugins section and Deactivate/Activate the plugin to be sure the structure of database has been checked, and corrected if needed. If the issue persist, please <a href="https://cff.dwbooster.com/contact-us">contact us</a></div>', 'calculated-fields-form' );
			}
		}
		else
		{
			$error_occur = true;
		}
	}
	else
    {
		$error_occur = true;
    }

	if( $error_occur )
	{
		global $cff_structure_error;
        $cff_structure_error = __('<div class="error-text">The data cannot be stored in database because has occurred an error with the form structure. Please, try to save the data again. If have been copied and pasted data from external text editors, the data can contain invalid characters. If the issue persist, please <a href="https://cff.dwbooster.com/contact-us">contact us</a></div>', 'calculated-fields-form' );
	}
}
?>